using System;
using System.IO;
using System.Collections;

using Syncfusion.Calculate;
using Syncfusion.XlsIO;

namespace XlsIOSamples
{
	/// <summary>
	/// A CalcWorkbook-derived object that uses XlsIO to read / compute Excel XLS files.
	/// </summary>
	public class XlsIOCalcWorkbook : CalcWorkbook
	{
		/// <summary>
		/// The XlsIO IWorkbook used by this class.
		/// </summary>
		public IWorkbook XlsIOWB;
		 
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="calcSheets">Array of XlsIOCalcSheet objects used in this workbook.</param>
		/// <param name="namedRanges">A hashtable containing named ranges.</param>
		public XlsIOCalcWorkbook(XlsIOCalcSheet[]calcSheets, Hashtable namedRanges) 
			: base(calcSheets, namedRanges)
		{

		}

		/// <summary>
		/// Creates a XlsIOCalcWorkbook object from an XLS file.
		/// </summary>
		/// <param name="fileName">Complete pathname of the XLS file.</param>
		/// <returns></returns>
		public static XlsIOCalcWorkbook CreateFromXLS(string fileName)
		{

			//Step 1 : Instantiate the spreadsheet creation engine.
			ExcelEngine excelEngine = new ExcelEngine();
			//Step 2 : Instantiate the excel application object.
			IApplication application = excelEngine.Excel;
			
			//No exception will be thrown when workbook is now saved.
			excelEngine.ThrowNotSavedOnDestroy = true;

			IWorkbook wb;
			try
			{

				wb = application.Workbooks.Open(fileName);
			}
			catch
			{
				throw new FileLoadException("XlsIO cannot load the file.", fileName);
			}

			XlsIOCalcSheet[] sheets = new XlsIOCalcSheet[wb.Worksheets.Count];

			for(int i = 0; i < wb.Worksheets.Count; ++i)
			{
				sheets[i] = new XlsIOCalcSheet(wb.Worksheets[i]);
				sheets[i].Name = wb.Worksheets[i].Name;
				
			}

			Hashtable ranges = new Hashtable();
			foreach(IName name in wb.Names)
			{
				ranges.Add(name.Name.ToUpper(), name.Value.Replace("'", ""));
			}

			XlsIOCalcWorkbook cwb = new XlsIOCalcWorkbook(sheets, ranges);
			return cwb;


		}


		/// <summary>
		/// Creates a ExceRWCalcWorkbook object from an IWorkbook.
		/// </summary>
		/// <param name="fileName">XlsIO workbook</param>
		/// <returns></returns>
		public static XlsIOCalcWorkbook CreateFromIWorkbook(IWorkbook wBook)
		{
			IWorkbook wb = wBook;

			XlsIOCalcSheet[] sheets = new XlsIOCalcSheet[wb.Worksheets.Count];

			for(int i = 0; i < wb.Worksheets.Count; ++i)
			{
				sheets[i] = new XlsIOCalcSheet(wb.Worksheets[i]);
				sheets[i].Name = wb.Worksheets[i].Name;
				
			}

			Hashtable ranges = new Hashtable();
			foreach(IName name in wb.Names)
			{
				ranges.Add(name.Name.ToUpper(), name.Value.Replace("'", ""));
			}

			XlsIOCalcWorkbook cwb = new XlsIOCalcWorkbook(sheets, ranges);
			return cwb;

		}

		/// <summary>
		/// Performs all calculations in the workbook.
		/// </summary>
		public override void CalculateAll()
		{
			foreach(CalcSheet sheet in this.calcSheets)
			{
				sheet.CalculationsSuspended = false;
			}

			foreach(XlsIOCalcSheet sheet in this.calcSheets)
			{
				for(int row = 1; row <= sheet.RowCount; ++row)
				{
					for(int col = 1; col <= sheet.ColCount; ++col)
					{
						object o = sheet[row, col];
						if(o != null)
						{
							string s2 = o.ToString();
							if(s2.Length > 0 && s2[0] == '=')
								sheet[row, col] = s2;
						}
					}
					
				}
			}
		}
	}

	/// <summary>
	/// Specially derived CalcSheet for use with XlsIO.
	/// </summary>
	public class XlsIOCalcSheet : CalcSheet
	{
		IWorksheet excelSheet;

		/// <summary>
		/// Constructor
		/// </summary>
		public XlsIOCalcSheet()
		{
			excelSheet = null;
		}

		/// <summary>
		/// Constuctor
		/// </summary>
		/// <param name="sheet">Underlying XlsIO IWorksheet object.</param>
		public XlsIOCalcSheet(IWorksheet sheet)
		{
			excelSheet = sheet;
		}

		/// <summary>
		/// Sets a value into the XlsIO worksheet.
		/// </summary>
		/// <param name="rowPos">The row index of the changed value.</param>
		/// <param name="colPos">The column index of the changed value.</param>
		/// <param name="val">The new value.</param>
		/// <remarks>
		/// If CalculationsSuspended is not True, a ValueChanged event is raised.
		/// </remarks>
		public override void SetValue(int rowPos, int colPos, string val)
		{

			//SetValueRowCol(val, rowPos, colPos);
			if(CalculationsSuspended)
				return;

			
			ValueChangedEventArgs e1 = new ValueChangedEventArgs(rowPos, colPos, val);
			base.OnValueChanged(e1);
		}

		/// <summary>
		/// Number of rows in this spreadsheet.
		/// </summary>
		public new int RowCount
		{
			get{return this.excelSheet.UsedRange.End.Row;}
		}

		/// <summary>
		/// The number of columns in this spreadsheet.
		/// </summary>
		public new int ColCount
		{
			get{return this.excelSheet.UsedRange.End.Column;}
		}

		 
		/// <summary>
		/// Returns the value at the specified row and column.
		/// </summary>
		/// <param name="row">Row index (one-based).</param>
		/// <param name="col">Column index (one-based).</param>
		/// <returns>The value at the specified row and column.</returns>
		public override object GetValueRowCol(int row, int col)
		{
			object o = excelSheet[row, col].Formula;
			if(o == null)
				o = excelSheet[row, col].Value;
			if(o != null)
				return o.ToString().Replace("'", "");
			return o;
		}

		/// <summary>
		/// Set the value at a specified row and column.
		/// </summary>
		/// <param name="value">The value to be set.</param>
		/// <param name="rowPos">The row index (one-based).</param>
		/// <param name="colPos">The column index (one-based).</param>
		public override void SetValueRowCol(object value, int row, int col)
		{
			excelSheet[row, col].FormulaNumberValue = Double.Parse(value.ToString());
		}
	}

}
